<div class="navbar navbar-fixed-bottom navbar-inverse">
  <div class="navbar-inner">
    <div class="container text-center">
        <ul class="nav">
          <li>
            <p class="navbar-text" style="color:white;"><%= product_version %></p>
          </li>
          <li>
            <p class="navbar-text redis-url" title="<%= redis_connection_and_namespace %>"><%= redis_connection_and_namespace %></p>
          </li>
          <li>
            <p class="navbar-text"><%= Time.now.utc.strftime('%H:%M:%S UTC') %></p>
          </li>
        </ul>
    </div>
  </div>
</div>
