// Czech
jQuery.timeago.settings.strings = {
  prefixAgo: "před",
  prefixFromNow: null,
  suffixAgo: null,
  suffixFromNow: null,
  seconds: "méně než minutou",
  minute: "minutou",
  minutes: "%d minutami",
  hour: "hodinou",
  hours: "%d hodinami",
  day: "1 dnem",
  days: "%d dny",
  month: "1 měsícem",
  months: "%d měsíci",
  year: "1 rokem",
  years: "%d roky"
};