require_relative 'test_helper'
<% module_namespacing do -%>
class <%= class_name %>WorkerTest < MiniTest::Unit::TestCase
    def test_example
      skip "add some examples to (or delete) #{__FILE__}"
    end
end
<% end -%>