<!doctype html>
<html>
  <head>
    <title><%= environment_title_prefix %><%= Sidekiq::NAME %></title>
    <meta name="viewport" content="width=device-width,initial-scale=1.0" />
    <link href="<%= root_path %>stylesheets/bootstrap.css" media="screen" rel="stylesheet" type="text/css" />
    <link href="<%= root_path %>stylesheets/application.css" media="screen" rel="stylesheet" type="text/css" />
    <link rel="shortcut icon" type="image/ico" href="<%= root_path %>images/favicon.ico" />
    <script type="text/javascript" src="<%= root_path %>javascripts/application.js"></script>
    <script type="text/javascript" src="<%= root_path %>javascripts/locales/jquery.timeago.<%= locale %>.js"></script>
    <meta name="google" content="notranslate" />
    <%= display_custom_head %>
  </head>
  <body class="admin">
    <%= erb :_nav %>
    <div id="page">
      <div class="container">
        <div class="row">
          <div class="col-sm-12 summary_bar">
            <%= erb :_summary %>
          </div>

          <div class="col-sm-12">
            <%= yield %>
          </div>
        </div>
      </div>
    </div>
    <%=  erb :_footer %>
    <%= erb :_poll_js %>
  </body>
</html>
