// Traditional Chinese, zh-tw
jQuery.timeago.settings.strings = {
  prefixAgo: null,
  prefixFromNow: "從現在開始",
  suffixAgo: "之前",
  suffixFromNow: null,
  seconds: "不到 1 分鐘",
  minute: "大約 1 分鐘",
  minutes: "%d 分鐘",
  hour: "大約 1 小時",
  hours: "%d 小時",
  day: "大約 1 天",
  days: "%d 天",
  month: "大約 1 個月",
  months: "%d 個月",
  year: "大約 1 年",
  years: "%d 年",
  numbers: [],
  wordSeparator: ""
};
