// Hebrew
jQuery.timeago.settings.strings = {
  prefixAgo: "לפני",
  prefixFromNow: "מעכשיו",
  suffixAgo: "",
  suffixFromNow: "",
  seconds: "פחות מדקה",
  minute: "דקה",
  minutes: "%d דקות",
  hour: "שעה",
  hours: "%d שעות",
  day: "יום",
  days: "%d ימים",
  month: "חודש",
  months: "%d חודשים",
  year: "שנה",
  years: "%d שנים"
};